@@ -14,7 +14,7 @@ from typing import TYPE_CHECKING, Any, AnyStr, ForwardRef, NewType, TypeVar, Uni
 
 from docutils import nodes
 from docutils.frontend import get_default_settings
-from sphinx.ext.autodoc.mock import mock
+from sphinx.ext.autodoc.mock import mock  # type: ignore[attr-defined]
 from sphinx.parsers import RSTParser
 from sphinx.util import logging, rst
 from sphinx.util.inspect import TypeAliasForwardRef, stringify_signature
@@ -36,7 +36,17 @@ if TYPE_CHECKING:
     from sphinx.ext.autodoc import Options
 
 _LOGGER = logging.getLogger(__name__)
-_PYDATA_ANNOTS_TYPING = {"Any", "AnyStr", "Callable", "ClassVar", "Literal", "NoReturn", "Optional", "Tuple", "Union"}
+_PYDATA_ANNOTS_TYPING = {
+    "Any",
+    "AnyStr",
+    "Callable",
+    "ClassVar",
+    "Literal",
+    "NoReturn",
+    "Optional",
+    "Tuple",
+    *({"Union"} if sys.version_info < (3, 14) else set()),
+}
 _PYDATA_ANNOTS_TYPES = {
     *("AsyncGeneratorType", "BuiltinFunctionType", "BuiltinMethodType"),
     *("CellType", "ClassMethodDescriptorType", "CoroutineType"),
@@ -246,8 +256,10 @@ def format_annotation(annotation: Any, config: Config, *, short_literals: bool =
     formatted_args: str | None = ""
 
     always_use_bars_union: bool = getattr(config, "always_use_bars_union", True)
-    is_bars_union = full_name == "types.UnionType" or (
-        always_use_bars_union and type(annotation).__qualname__ == "_UnionGenericAlias"
+    is_bars_union = (
+        (sys.version_info >= (3, 14) and full_name == "typing.Union")
+        or full_name == "types.UnionType"
+        or (always_use_bars_union and type(annotation).__qualname__ == "_UnionGenericAlias")
     )
     if is_bars_union:
         full_name = ""
@@ -255,7 +267,10 @@ def format_annotation(annotation: Any, config: Config, *, short_literals: bool =
     # Some types require special handling
     if full_name == "typing.NewType":
         args_format = f"\\(``{annotation.__name__}``, {{}})"
-        role = "class"
+        role = "obj"
+    elif full_name == "typing.Annotated":
+        # By default we don't show metadata in Annotated
+        return format_annotation(annotation.__origin__, config, short_literals=short_literals)
     elif full_name in {"typing.TypeVar", "typing.ParamSpec"}:
         params = {k: getattr(annotation, f"__{k}__") for k in ("bound", "covariant", "contravariant")}
         params = {k: v for k, v in params.items() if v}
@@ -288,6 +303,8 @@ def format_annotation(annotation: Any, config: Config, *, short_literals: bool =
             return f"\\{' | '.join(f'``{arg!r}``' for arg in args)}"
         formatted_args = f"\\[{', '.join(f'``{arg!r}``' for arg in args)}]"
     elif is_bars_union:
+        if not args:
+            return f":py:{'class' if sys.version_info >= (3, 14) else 'data'}:`{prefix}typing.Union`"
         return " | ".join([format_annotation(arg, config, short_literals=short_literals) for arg in args])
 
     if args and not formatted_args:
@@ -347,7 +364,7 @@ def normalize_source_lines(source_lines: str) -> str:
     return "\n".join(aligned_prefix + aligned_suffix)
 
 
-def process_signature(  # noqa: C901, PLR0913, PLR0917
+def process_signature(  # noqa: C901, PLR0911, PLR0912, PLR0913, PLR0917
     app: Sphinx,
     what: str,
     name: str,
@@ -376,7 +393,10 @@ def process_signature(  # noqa: C901, PLR0913, PLR0917
     if not getattr(obj, "__annotations__", None):  # when has no annotation we cannot autodoc typehints so bail
         return None
 
-    obj = inspect.unwrap(obj)
+    try:
+        obj = inspect.unwrap(obj)
+    except ValueError:
+        return None
     sph_signature = sphinx_signature(obj, type_aliases=app.config["autodoc_type_aliases"])
     typehints_formatter: Callable[..., str | None] | None = getattr(app.config, "typehints_formatter", None)
 
@@ -407,11 +427,19 @@ def process_signature(  # noqa: C901, PLR0913, PLR0917
         elif what == "method":
             # bail if it is a local method as we cannot determine if first argument needs to be deleted or not
             if "<locals>" in obj.__qualname__ and not _is_dataclass(name, what, obj.__qualname__):
-                _LOGGER.warning('Cannot handle as a local function: "%s" (use @functools.wraps)', name)
+                _LOGGER.warning(
+                    'Cannot handle as a local function: "%s" (use @functools.wraps)',
+                    name,
+                    type="sphinx_autodoc_typehints",
+                    subtype="local_function",
+                )
                 return None
             outer = inspect.getmodule(obj)
+            if outer is None:
+                return None
             for class_name in obj.__qualname__.split(".")[:-1]:
-                outer = getattr(outer, class_name)
+                if (outer := getattr(outer, class_name, None)) is None:
+                    return None
             method_name = obj.__name__
             if method_name.startswith("__") and not method_name.endswith("__"):
                 # when method starts with double underscore Python applies mangling -> prepend the class name
@@ -500,7 +528,9 @@ def _execute_guarded_code(autodoc_mock_imports: list[str], obj: Any, module_code
                     with mock(autodoc_mock_imports):
                         exec(guarded_code, getattr(obj, "__globals__", obj.__dict__))  # noqa: S102
         except Exception as exc:  # noqa: BLE001
-            _LOGGER.warning("Failed guarded type import with %r", exc)
+            _LOGGER.warning(
+                "Failed guarded type import with %r", exc, type="sphinx_autodoc_typehints", subtype="guarded_import"
+            )
 
 
 def _resolve_type_guarded_imports(autodoc_mock_imports: list[str], obj: Any) -> None:
@@ -527,7 +557,7 @@ def _get_type_hint(
 ) -> dict[str, Any]:
     _resolve_type_guarded_imports(autodoc_mock_imports, obj)
     try:
-        result = get_type_hints(obj, None, localns)
+        result = get_type_hints(obj, None, localns, include_extras=True)
     except (AttributeError, TypeError, RecursionError) as exc:
         # TypeError - slot wrapper, PEP-563 when part of new syntax not supported
         # RecursionError - some recursive type definitions https://github.com/python/typing/issues/574
@@ -536,7 +566,13 @@ def _get_type_hint(
         else:
             result = {}
     except NameError as exc:
-        _LOGGER.warning('Cannot resolve forward reference in type annotations of "%s": %s', name, exc)
+        _LOGGER.warning(
+            'Cannot resolve forward reference in type annotations of "%s": %s',
+            name,
+            exc,
+            type="sphinx_autodoc_typehints",
+            subtype="forward_reference",
+        )
         result = obj.__annotations__
     return result
 
@@ -554,7 +590,13 @@ def backfill_type_hints(obj: Any, name: str) -> dict[str, Any]:  # noqa: C901, P
     def _one_child(module: Module) -> stmt | None:
         children = module.body  # use the body to ignore type comments
         if len(children) != 1:
-            _LOGGER.warning('Did not get exactly one node from AST for "%s", got %s', name, len(children))
+            _LOGGER.warning(
+                'Did not get exactly one node from AST for "%s", got %s',
+                name,
+                len(children),
+                type="sphinx_autodoc_typehints",
+                subtype="multiple_ast_nodes",
+            )
             return None
         return children[0]
 
@@ -569,7 +611,7 @@ def backfill_type_hints(obj: Any, name: str) -> dict[str, Any]:  # noqa: C901, P
         return {}
 
     try:
-        type_comment = obj_ast.type_comment
+        type_comment = obj_ast.type_comment  # type: ignore[attr-defined]
     except AttributeError:
         return {}
 
@@ -579,14 +621,19 @@ def backfill_type_hints(obj: Any, name: str) -> dict[str, Any]:  # noqa: C901, P
     try:
         comment_args_str, comment_returns = type_comment.split(" -> ")
     except ValueError:
-        _LOGGER.warning('Unparseable type hint comment for "%s": Expected to contain ` -> `', name)
+        _LOGGER.warning(
+            'Unparseable type hint comment for "%s": Expected to contain ` -> `',
+            name,
+            type="sphinx_autodoc_typehints",
+            subtype="comment",
+        )
         return {}
 
     rv = {}
     if comment_returns:
         rv["return"] = comment_returns
 
-    args = load_args(obj_ast)
+    args = load_args(obj_ast)  # type: ignore[arg-type]
     comment_args = split_type_comment_args(comment_args_str)
     is_inline = len(comment_args) == 1 and comment_args[0] == "..."
     if not is_inline:
@@ -594,7 +641,9 @@ def backfill_type_hints(obj: Any, name: str) -> dict[str, Any]:  # noqa: C901, P
             comment_args.insert(0, None)  # self/cls may be omitted in type comments, insert blank
 
         if len(args) != len(comment_args):
-            _LOGGER.warning('Not enough type comments found on "%s"', name)
+            _LOGGER.warning(
+                'Not enough type comments found on "%s"', name, type="sphinx_autodoc_typehints", subtype="comment"
+            )
             return rv
 
     for at, arg in enumerate(args):
@@ -689,7 +738,10 @@ def process_docstring(  # noqa: PLR0913, PLR0917
     if not callable(obj):
         return
     obj = obj.__init__ if inspect.isclass(obj) else obj
-    obj = inspect.unwrap(obj)
+    try:
+        obj = inspect.unwrap(obj)
+    except ValueError:
+        return
 
     try:
         signature = sphinx_signature(obj, type_aliases=app.config["autodoc_type_aliases"])
@@ -768,43 +820,63 @@ def _inject_signature(
     lines: list[str],
 ) -> None:
     for arg_name in signature.parameters:
-        annotation = type_hints.get(arg_name)
+        _inject_arg_signature(type_hints, signature, app, lines, arg_name)
 
-        default = signature.parameters[arg_name].default
 
-        if arg_name.endswith("_"):
-            arg_name = f"{arg_name[:-1]}\\_"  # noqa: PLW2901
+def _inject_arg_signature(
+    type_hints: dict[str, Any],
+    signature: inspect.Signature,
+    app: Sphinx,
+    lines: list[str],
+    arg_name: str,
+) -> None:
+    annotation = type_hints.get(arg_name)
 
-        insert_index = None
-        for at, line in enumerate(lines):
-            if _line_is_param_line_for_arg(line, arg_name):
-                # Get the arg_name from the doc to match up for type in case it has a star prefix.
-                # Line is in the correct format so this is guaranteed to return tuple[str, str].
-                func = _get_sphinx_line_keyword_and_argument
-                _, arg_name = func(line)  # type: ignore[assignment, misc] # noqa: PLW2901
-                insert_index = at
-                break
+    default = signature.parameters[arg_name].default
 
-        if annotation is not None and insert_index is None and app.config.always_document_param_types:
-            lines.append(f":param {arg_name}:")
-            insert_index = len(lines)
+    if arg_name.endswith("_"):
+        arg_name = f"{arg_name[:-1]}\\_"
 
-        if insert_index is not None:
-            if annotation is None:
-                type_annotation = f":type {arg_name}: "
-            else:
-                short_literals = app.config.python_display_short_literal_types
-                formatted_annotation = add_type_css_class(
-                    format_annotation(annotation, app.config, short_literals=short_literals)
-                )
-                type_annotation = f":type {arg_name}: {formatted_annotation}"
+    insert_index = None
+    for at, line in enumerate(lines):
+        if _line_is_param_line_for_arg(line, arg_name):
+            # Get the arg_name from the doc to match up for type in case it has a star prefix.
+            # Line is in the correct format so this is guaranteed to return tuple[str, str].
+            _, arg_name = _get_sphinx_line_keyword_and_argument(line)  # type: ignore[assignment, misc]
+            insert_index = at
+            break
 
-            if app.config.typehints_defaults:
-                formatted_default = format_default(app, default, annotation is not None)
-                if formatted_default:
-                    type_annotation = _append_default(app, lines, insert_index, type_annotation, formatted_default)
+    if annotation is not None and insert_index is None and app.config.always_document_param_types:
+        lines.append(f":param {arg_name}:")
+        insert_index = len(lines) - 1
 
-            lines.insert(insert_index, type_annotation)
+    if insert_index is not None:
+        has_preexisting_annotation = False
+
+        if annotation is None:
+            type_annotation, has_preexisting_annotation = _find_preexisting_type_annotation(lines, arg_name)
+        else:
+            short_literals = app.config.python_display_short_literal_types
+            formatted_annotation = add_type_css_class(
+                format_annotation(annotation, app.config, short_literals=short_literals)
+            )
+            type_annotation = f":type {arg_name}: {formatted_annotation}"
+
+        if app.config.typehints_defaults:
+            formatted_default = format_default(app, default, annotation is not None or has_preexisting_annotation)
+            if formatted_default:
+                type_annotation = _append_default(app, lines, insert_index, type_annotation, formatted_default)
+
+        lines.insert(insert_index, type_annotation)
+
+
+def _find_preexisting_type_annotation(lines: list[str], arg_name: str) -> tuple[str, bool]:
+    """Find a type entry in the input docstring that matches the given arg name."""
+    type_annotation = f":type {arg_name}: "
+    for line in lines:
+        if line.startswith(type_annotation):
+            return line, True
+    return type_annotation, False
 
 
 def _append_default(
@@ -877,7 +949,7 @@ def get_insert_index(app: Sphinx, lines: list[str]) -> InsertIndexInfo | None:
 
     # 3. Insert after the parameters.
     # To find the parameters, parse as a docutils tree.
-    settings = get_default_settings(RSTParser)
+    settings = get_default_settings(RSTParser)  # type: ignore[arg-type]
     settings.env = app.env
     doc = parse("\n".join(lines), settings)
 
